local function SafeFreeSlot(...)
	for _,slot in ipairs({...})
		if not rawget(_G, slot) freeslot(slot) end -- Ensure we don't accidentally overlap existing freeslots
	end
end
SafeFreeSlot("sfx_wl4sba", "sfx_wlskid", "sfx_wlsbwa", "sfx_wlatta") // SFX freeslot assignment likely requires we freeslot before assigning
sfxinfo[sfx_wl4sba].caption = 'Shoulder Bash'
sfxinfo[sfx_wlskid].caption = 'Skidding'
sfxinfo[sfx_wlsbwa].caption = 'Hit Wall'
sfxinfo[sfx_wlatta].caption = 'Hit Enemy'

sfxinfo[sfx_wlatta] = {
        singular = false,
        priority = 1,
        flags = SF_X2AWAYSOUND
}

sfxinfo[sfx_wlsbwa] = {
        singular = false,
        priority = 1,
        flags = SF_X2AWAYSOUND
}

/// Ability: Punch
// allows you to do a thrusting melee attack

// CUSTOMIZE SECTION START
local skin = "kombi" // your character here
local thebutton = BT_CUSTOM2 // the button needed to do the punch
local sound = sfx_wl4sba // the sound of the punch
local punchlenght = 55 // the lenght of the punch
local hasdust = false // if this is true, you will spawn dust while punching
local hasghost = true // if this is true, you will have a after effect while punching
local hasthok = false  // if this true, you will have a thok effect while punching
local punchdelay = 10 // the delay of punching
local cooldownhud = false // if this is true, the cooldown will be shown in a heads up display
// CUSTOMIZE SECTION END PROCCED BELOW IF YOU KNOW WHAT YOUR DOING

freeslot("S_PLAY_PUNCH","SPR2_BASH")
// can modifiy this for your own punch frames if you want
states[S_PLAY_PUNCH] = {
	sprite = SPR_PLAY,
	frame = A|SPR2_BASH|FF_ANIMATE,
	tics = punchlenght,
	var1 = 3,
	var2 = 4,
	nextstate = S_PLAY_PUNCH
}

addHook("PlayerThink", function(player)
	if not player.punchwait
		player.punchwait = 0
	end
	if not player.realmo.headbashing
		player.realmo.headbashing = 0
	end
	if player.punchwait
		player.punchwait = $ - 1
	end
	local punchspeed = (skins[player.realmo.skin].normalspeed*4)/3
	if player.realmo.punching == nil
		player.realmo.punching = false
	end
	if player.realmo.punchtime == nil
		player.realmo.punchtime = 0
	end
	if player.realmo.punchtime > 0
		player.realmo.punching = true
		if player.realmo.pause <= 0
			player.realmo.punchtime = $ - 1
		end
		if not player.realmo.headbashing and player.realmo.pause <= 0
			player.pflags = $|PF_STASIS
			player.realmo.state = S_PLAY_PUNCH
			player.drawangle = player.realmo.punchangle
			if not srb2p
				P_InstaThrust(player.realmo, player.drawangle, punchspeed)
			else
				P_InstaThrust(player.realmo, player.drawangle, 3*punchspeed/4)
			end
		elseif player.realmo.headbashing
			if (player.pflags&PF_STASIS)
				player.pflags = $&~PF_STASIS
			end
			if abs(player.realmo.momx) + abs(player.realmo.momy)*((player.realmo.eflags & MFE_UNDERWATER)>>2 or 1) >= 3*player.normalspeed/4
				player.realmo.punchtime = 35
			end
			if player.realmo.punchtime == 1
				player.realmo.punchtime = 0
				player.realmo.punching = false
				player.realmo.headbashing = 0
				if not srb2p
					player.powers[pw_strong] = STR_NONE
				end
			end
		end
		if not srb2p
			player.powers[pw_strong] = STR_BUST|STR_SPIKE|STR_PUNCH|STR_ANIM
		end
		if hasdust == true
			P_SpawnSkidDust(player, player.realmo.radius, true)
		end
		if player.realmo.headbashing and leveltime%3 == 0
			local punchtrail = P_SpawnGhostMobj(player.realmo)
			punchtrail.colorized = true
			punchtrail.color = (leveltime/9%79)+31
			punchtrail.fuse = 9 
			punchtrail.blendmode = AST_ADD 
		end
		if hasthok == true
			local thokeffect = P_SpawnThokMobj(player) 
		end
	end
	if player.realmo.punchtime == 1
		if not srb2p
			player.powers[pw_strong] = STR_NONE
		end
		player.realmo.punching = false
		player.nospindash = false
		player.realmo.headbashing = 0
		player.pflags = $1 & ~PF_FULLSTASIS
		player.punchwait = punchdelay
	end
	if player.realmo and player.realmo.skin == skin
	and P_PlayerInPain(player)
	or P_CheckDeathPitCollide(player.realmo)
	or not (player.playerstate == PST_LIVE)
	and player.realmo.punching == true
		if not srb2p
			player.powers[pw_strong] = STR_NONE
		end
		player.pflags = $1 & ~PF_FULLSTASIS
		player.realmo.punching = false
		player.nospindash = false
		player.realmo.headbashing = 0
		player.realmo.punchtime = 0
	end
	if player.realmo.state == S_PLAY_PUNCH
	and player.realmo.punchtime == 0
		player.realmo.punching = false
		player.nospindash = false
		player.realmo.headbashing = 0
		player.realmo.state = S_PLAY_RUN
		if not srb2p
			player.powers[pw_strong] = STR_NONE
		end
	end
	if player.realmo and player.realmo.skin == skin
	and (player.cmd.buttons & thebutton)
	and P_IsObjectOnGround(player.realmo)
	and player.realmo.punching == false
	and player.realmo.punchtime == 0
	and not player.punchwait
		player.drawangle = player.realmo.angle
		player.realmo.punchangle = player.drawangle
		S_StartSound(player.realmo, sound)
		player.realmo.punchtime = punchlenght
		player.realmo.state = S_PLAY_PUNCH
	elseif player.realmo and player.realmo.skin == skin
	and (player.cmd.buttons & thebutton)
	and (player.realmo.punchtime < (punchlenght - 10) and not player.realmo.headbashing)
	and not player.punchwait
		player.realmo.headbashing = 1
		player.speedinc = FRACUNIT*24
		player.punchwait = 6
		player.realmo.state = S_PLAY_DASH
	end
end)

addHook("MobjDamage", function(target, inf, src, dmg, dmgType)
	if inf == nil or inf.player == nil then return end
	if inf.punching and target.flags & MF_ENEMY
		S_StartSound(inf, sfx_wlatta)
	end
end)